function [hiddenX, obsE] = sample_HMM(initial_prob, transmatP, obsmatE, T, numbex)
% Sampling of data for a given HMM structure.

% sample_HMM produces a realization of a Hidden Markov Model with transition probability
% matrix transmatP, and observation emission matrix obsmatE.
% The vector initial_prob is used as the initial state, and there are
% generated numbex sequenses both of the hidden chain and the related oberved sequence.
% Each of the numbex row of hiddenX and obsE is of length T.

% From the mendelHMM-toolbox by Steinar Thorvaldsen, 2004.
% http://www.math.uit.no/bi/hmm/

% [hiddenX, obsE] = sample_HMM(initial_prob, transmatP, obsmatE, T, numbex)
       
% Outputs: hiddenX  =   Matrix of sequence of dim (mumbex x T) of numbers between 1 and n, 
%                       where n is the number of states or the dimension of
%                       transmatP
%          obsE =       Matrix of sequences of dim (numbex x T) of numbers
%                       between 1 and m, where m is given by the dim of obsmatE.

% Inputs:  initial_prob = Initial state vector.
%          transmatP =    Transition probability matrix
%          obsmatE =      Observation emission matrix
%          T  =           The length of time series
%          numbex =       The number of chains         
%

hiddenX = sample_MC(initial_prob, transmatP, T, numbex);
obsE = zeros(numbex, T);

% We may now use the general function  sample_discr  to compute the related
% observation for each state:
for i=1:numbex
  h = hiddenX(i,1);
  obsE(i,1) = sample_discr(obsmatE(h,:));
  for j=2:T
    h = hiddenX(i,j);
    obsE(i,j) = sample_discr(obsmatE(h,:));
  end
end