function  [obsE] = resample_mendeldata (experiment)
% Resampling of the observervational data from Mendel's different experiments (1..9).
% Ref.: Stern, C and. Sherwood, E.R.(1966), The Origin of genetics: a
% Mendel source book. pp 10-15 (experiment 1-7), pp 17-19 (emperiment 8)
% and pp 20-21 (emperiment 9).
% In experiment 1-7, only about 10% of the dominants in F2 were tested
% future in F3, and we have restricted our study to this subset.
% The resampled results are stored in the observation matrix called dataE.

% From the mendelHMM-toolbox by Steinar Thorvaldsen,
% http://www.math.uit.no/bi/hmm/

% experiment: the number of the dataset to restore
% dataE: the matrix containing the observed phenotype data

switch experiment
    case 1 % exp. 1 with 1 gene
        freq = [756 565 472
                  0 191 284];
    case 2 % exp. 2 with 1 gene
        freq = [691 519 431
                  0 172 260];
    case 3 % exp. 3 with 1 gene..
        freq = [132 100 84
                  0  32 48];
    case 4
        freq = [134 100 82
                  0  34 52];          
    case 5
        freq = [136 100 85
                  0  36 51];
    case 6
        freq = [132 100 83
                  0  32 49];
    case 7
        freq = [135 100 82
                  0  35 53];
    case 8 % the exp. with 2 genes
        freq = [529 301 209
                  0  96 120
                  0 102 127
                  0  30  73];
    case 9 % the exp. with 3 genes
        freq = [639 269 153
                  0  88  92
                  0  86  92
                  0  30  56
                  0  98 100
                  0  34  60
                  0  27  55
                  0   7  31];
    end % switch
    
obsE= ones(freq(1,1),3); % default values. The observation series were for 3 generations
Enum=size(freq,1); % the number of phenotypes to populate the obsE matrix
    
indexsum2gen=freq(1,2);
indexsum3gen=freq(1,3);
for n=2:Enum
    index2span=indexsum2gen+freq(n,2);
    index3span=indexsum3gen+freq(n,3);
    obsE(indexsum2gen+1:index2span ,2)=n; % updating phenotype data in second generation
    obsE(indexsum3gen+1:index3span ,3)=n; % updating data in third generation
    indexsum2gen=index2span;
    indexsum3gen=index3span;
end %for

% It is not necessary to perform a random permutation 
% of the line elements in the obsE matrix to estimate the HMM. 
        
   