function  [loglik] = init_diagonal (X, E, dominance)
% a initialization of the matrixes in done here by the method of diagonal dominance.
% The results are stored in the global variables with names ending with 1.
% From the mendelHMM-toolbox by Steinar Thorvaldsen,
% http://www.math.uit.no/bi/hmm/

% E: number of observables
% X: number of states
% dominance must be between 0 and 1

global dataE prior prior1 transmat1 obsmat1 
g=log(E)/log(2); %the number of genes
X1g=X^(1/g); %the umber of states in the 1-gene case (3 for diploid, 5 for tetraploid)
loglik = -Inf;
prior1 = prior %The Mendel startup state (heterozygote plants)

% 1. The basic transition matrix for 1 gene:
transmat1 = ones(X1g,X1g);  %makes each element=1 in the matrix.
minorX1 = (1-dominance)/(X1g-1); % the fractions that are left to be outside the diagonal in the matrix lines
transmat1 = transmat1*minorX1; %populates the matrixes with the minor elements
% Then we populates the diagonals in the transmat1 for 1 gene with the dominant density:
for i= 1:X1g
    transmat1(i,i)=dominance;
end
% If the model have 2 or 3 genes, the matrices can be constructed
% iteratively from the 1 gene case by utilizing independence between the genes.
% The process of joining small matrices to make bigger ones is done by
% so called concatenation. (The code here is in no way as elegant as the mathematics are...)
if g > 1
    for gene= 2:g
        nullmat=zeros(size(transmat1));
        if X1g==3 %diploid case
            transmat1 = [transmat1  nullmat nullmat; transmat1*minorX1 transmat1*dominance transmat1*minorX1; nullmat nullmat transmat1];
        end
        if X1g==5 %tetraploid case
            transmat1 = [transmat1  nullmat nullmat nullmat nullmat; transmat1*minorX1*2 transmat1*dominance transmat1*minorX1*2 nullmat nullmat; transmat1*minorX1 transmat1*minorX1 transmat1*dominance transmat1*minorX1 transmat1*minorX1; nullmat nullmat transmat1*minorX1*2 transmat1*dominance transmat1*minorX1*2 ; nullmat nullmat nullmat nullmat transmat1];    
        end %other cases must be implemented seperately       
    end
end

% 2. The observation (emission) matrix is not square.
% First we constructe it for 1 gene, and then iteratively for 2 or 3 genes.
% The obsmat for 1 gene we diagonalize from both ends, and then connect the ends.

E1g=2; %the default number of observations for 1 gene
minorE = (1-dominance)/(E-1); % the fractions that are left to be outside the "diagonal"
obsmat1 = ones(X1g,E1g); 
obsmat1 = obsmat1*minorE;
% Making the "diagonal" dominant:
for i= 1:E1g/2
    obsmat1(i,i)=dominance;
    obsmat1(X1g+1-i,E1g+1-i)=dominance;
end
connectlength=X1g-E1g;
for j= E1g/2 +1 : X1g-i
    obsmat1(j,E1g/2)=dominance;
end
% If we have 2 or 3 genes, the matrix can be constructed
% iteratively from the 1 gene case.
% The process of joining small matrices to make bigger ones is done by
% concatenation. 
if g > 1
    for i = 2:g
        minormatObs=ones(size(obsmat1));
        minormatObs=minorE*minormatObs;
        if X1g==3 %diploid case
            obsmat1 = [obsmat1 minormatObs; obsmat1 minormatObs; minormatObs obsmat1];
        end
        if X1g==5 %tetraploid case
            obsmat1 = [obsmat1 minormatObs; obsmat1 minormatObs; obsmat1 minormatObs; obsmat1 minormatObs; minormatObs obsmat1];
        end %other cases must be implemented seperately
    end
end

% obsmat1=[d m 0 0; d m 0 0; m d 0 0; d m 0 0; d m 0 0; m d 0 0; 0 0 d m; 0 0 d m; 0 0 m d]
transmat1
obsmat1
loglik = HMM_logprob(dataE, prior1, transmat1, obsmat1);


