function  [loglik] = init_random (X, E)
% a random initial guess of parameters is done.
% Should be run several times to try avoiding lokal maxima
% If guess_rounds are to be three times the parameters in the model:
% guess_rounds = 3*(X + X*E + X^2);

% From the mendelHMM-toolbox by Steinar Thorvaldsen,
% http://www.math.uit.no/bi/hmm/

% E: number of observables
% X: number of states

global dataE prior prior1 transmat1 obsmat1 

prior1 = prior %The Mendel startup state (heterozygote plants)
loglik = -Inf;

g=log(E)/log(2); %the number of genes
X1g=X^(1/g); %the umber of states in the 1-gene case (3 for diploid, 5 for tetraploid)

% 1. The transition matrix:
transmat1 = rand(X,X);  %makes each element random in the matrix

% If the model has 2 or 3 genes, we utilize independence between the genes
% by setting parts of it to be zero:
if g > 1
    for i= 2:X1g
        transmat1(1:X1g^(g-1), X1g^(g-1)*(i-1)+1:X1g^(g-1)*i)=0;
        transmat1(X-X1g^(g-1)+1:X, X1g^(g-1)*(i-2)+1:X1g^(g-1)*(i-1))=0;
    end %other cases and details... can be implemented seperately  
end

transmat1 = normalise(transmat1,2);  %make each line in the matrixes  sum to 1 (in 2nd dimention)

% 2. The observation (emission) matrix:
obsmat1 = normalise(rand(X,E),2);   %obsmat1 = normalise(ones(X,E),2);
loglik = HMM_logprob(dataE, prior1, transmat1, obsmat1);

